﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Clients.Hive {
  [StorableClass]
  public partial class HiveExperiment : IDeepCloneable, IContent, IProgressReporter {
    [Storable]
    private bool useLocalPlugins;
    public bool UseLocalPlugins {
      get { return useLocalPlugins; }
      set { useLocalPlugins = value; }
    }

    [Storable]
    private ExecutionState executionState;
    public ExecutionState ExecutionState {
      get { return executionState; }
      internal set {
        if (executionState != value) {
          executionState = value;
          OnExecutionStateChanged();
        }
      }
    }

    [Storable]
    private TimeSpan executionTime;
    public TimeSpan ExecutionTime {
      get { return executionTime; }
      internal set {
        if (executionTime != value) {
          executionTime = value;
          OnExecutionTimeChanged();
        }
      }
    }

    [Storable]
    private ItemCollection<HiveJob> hiveJobs;
    public ItemCollection<HiveJob> HiveJobs {
      get { return hiveJobs; }
      set {
        if (hiveJobs != value) {
          hiveJobs = value;
          OnHiveJobsChanged();
        }
      }
    }

    [Storable]
    private bool isProgressing;
    public bool IsProgressing {
      get { return isProgressing; }
      set {
        if (isProgressing != value) {
          isProgressing = value;
          OnIsProgressingChanged();
        }
      }
    }

    [Storable]
    private IProgress progress;
    public IProgress Progress {
      get { return progress; }
      set { this.progress = value; }
    }

    #region Constructors and Cloning
    public HiveExperiment() {
      this.ResourceNames = "HEAL";
      this.HiveJobs = new ItemCollection<HiveJob>();
    }

    protected HiveExperiment(HiveExperiment original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      this.OwnerUserId = original.OwnerUserId;
      this.DateCreated = original.DateCreated;
      this.ResourceNames = original.ResourceNames;
      this.LastAccessed = original.LastAccessed;
      this.Name = original.Name;
      this.Description = original.Description;
      this.Id = original.Id;
      this.HiveJobs = cloner.Clone(original.HiveJobs);
      this.UseLocalPlugins = original.UseLocalPlugins;
      this.ExecutionTime = original.ExecutionTime;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new HiveExperiment(this, cloner);
    }
    #endregion

    #region Events
    public event EventHandler ExecutionTimeChanged;
    protected virtual void OnExecutionTimeChanged() {
      var handler = ExecutionTimeChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler ExecutionStateChanged;
    protected virtual void OnExecutionStateChanged() {
      var handler = ExecutionStateChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler HiveJobsChanged;
    protected virtual void OnHiveJobsChanged() {
      var handler = HiveJobsChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler IsProgressingChanged;
    protected virtual void OnIsProgressingChanged() {
      var handler = IsProgressingChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler Loaded;
    public virtual void OnLoaded() {
      var handler = Loaded;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    #endregion
    
    protected override void OnPropertyChanged(PropertyChangedEventArgs e) {
      base.OnPropertyChanged(e);
      if (e.PropertyName == "Name") {
        OnToStringChanged();
      }
    }
    
    protected override void RaisePropertyChanged(string propertyName) {
      if (!(propertyName == "ExecutionTime")
        && !(propertyName == "JobCount")
        && !(propertyName == "CalculatingCount")
        && !(propertyName == "FinishedCount")) {
        base.RaisePropertyChanged(propertyName);
      }
    }

    public bool IsFinished() {
      return HiveJobs != null
        && HiveJobs.All(x => x.Job.DateFinished.HasValue && x.Job.DateCreated.HasValue);
    }

    public IEnumerable<HiveJob> GetAllHiveJobs() {
      var jobs = new List<HiveJob>();
      foreach (HiveJob job in HiveJobs) {
        jobs.AddRange(job.GetAllHiveJobs());
      }
      return jobs;
    }

    public override string ToString() {
      return Name;
    }
  }
}
